/*	Copyright (c) 1989 Citadel	*/
/*	   All Rights Reserved    	*/

/* #ident	"rolodeck.c	1.2 - 89/10/31" */

#include <blkio.h>
#include <cbase.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#if __STDC__ == 1
#include <stdlib.h>
#include <string.h>
#include "rolodeck.h"
#else
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS	(0)
#define EXIT_FAILURE	(1)
#endif
#include "rolodeck.h"
#endif

/* function declarations */
#if __STDC__ == 1
int cvtss(char *t, const char *s, int m, size_t n);
int fmltolfm(char *t, const char *s, size_t n);
int lfmtofml(char *t, const char *s, size_t n);
#else
int cvtss();
int fmltolfm();
int lfmtofml();
#endif
#define CVT_X_SP	(0x02)		/* cvtss flags */
#define CVT_X_CTL	(0x04)
#define CVT_X_LEADSP	(0x08)
#define CVT_ONE_SP	(0x10)
#define CVT_X_TRAILSP	(0x80)

/* constants */
#define EXPFILE		("rolodeck.txt")	/* export file */
#define LMAXTRIES	(100)			/* maximum lock tries */

/* rolodeck user requests */
#define RD_REQ_NEXT_CARD	('N')	/* next card */
#define RD_REQ_PREV_CARD	('P')	/* previous card */
#define RD_REQ_FIRST_CARD	('F')	/* first card*/
#define RD_REQ_LAST_CARD	('L')	/* last card */
#define RD_REQ_INS_CARD		('I')	/* insert card */
#define RD_REQ_DEL_CARD		('D')	/* delete card */
#define RD_REQ_SRCH_CARD	('S')	/* search for card */
#define RD_REQ_TOG_SORT		('T')	/* toggle sort field */
#define RD_REQ_CUR_CARD		('C')	/* current card */
#define RD_REQ_ALL_CARDS	('A')	/* all cards */
#define RD_REQ_IMPORT		('M')	/* import cards */
#define RD_REQ_EXPORT		('X')	/* export cards */
#define RD_REQ_HELP		('H')	/* help */
#define RD_REQ_QUIT		('Q')	/* quit */

/*man---------------------------------------------------------------------------
NAME
    rolodeck - card file

SYNOPSIS
     rolodeck

DESCRIPTION
     rolodeck is an example program for the cbase library.  In order
     to allow it to be compiled without requiring a specific display
     library, only a minimal user interface has been implemented.

NOTES
     Below are listed a few of the more important points to note when
     examining the rolodeck source code.

          o White space is significant in string data.  For
            instance, " data" != "data".  Leading and
            trailing white space is therefore usually
            removed before storing a string in a database.
            Also, embedded white space may be reduced to a
            single space.  The cvtss function included with
            cbase will perform these string operations.
          o Names are often input and displayed
            first-name-first.  For them to sort correctly in
            a database, however, they must be stored
            last-name-first.  The functions fmltolfm and
            lfmtofml are included with cbase to convert
            between these two formats.
          o bexit is used in place of exit to prevent loss
            of buffered data.

     The following notes concern locking.

          o Single-tasking applications can simply lock
            a cbase and leave it locked.
          o Locks are held for shortest time possible; a
            lock is never held during user input.
          o A write lock should not be used when only a read
            lock is required.
          o When a database file is unlocked, it may be
            modified by another process.  A record at a
            given file position may be deleted, and the
            empty slot possibly reused for a new record.
            Because of this, each time a file is locked,
            the current record must be located by performing
            a search on a unique key.
          o If locking multiple cbases, deadlock must be
            avoided (see The cbase Programmer's Guide).

------------------------------------------------------------------------------*/
int main(argc, argv)
int argc;
char *argv[];
{
	cbase_t *cbp = NULL;	/* cbase pointer */
	int sf = 0;		/* sort field */
	int rq = 0;		/* user request */
	rolodeck_t rd;		/* cbase record */
	int found = 0;		/* search flag */
	char buf[256];		/* gp input buffer */

	/* open rolodeck cbase */
	cbp = cbopen(RDNAME, "r+", RDFLDC, rdfldv);
	if (cbp == NULL) {
		if (errno != ENOENT) {
			fprintf(stderr, "*** Error %d opening rolodeck.\n", errno);
			bexit(EXIT_FAILURE);
		}
		/* create rolodeck cbase */
		printf("Rolodeck does not exist.  Creating...\n");
		if (cbcreate(RDNAME, sizeof(rolodeck_t), RDFLDC, rdfldv) == -1) {
			fprintf(stderr, "*** Error %d creating rolodeck.\n", errno);
			bexit(EXIT_FAILURE);
		}
		cbp = cbopen(RDNAME, "r+", RDFLDC, rdfldv);
		if (cbp == NULL) {
			fprintf(stderr, "*** Error %d opening rolodeck.\n", errno);
			bexit(EXIT_FAILURE);
		}
	}
	if (cbrecsize(cbp) != sizeof(rolodeck_t)) {
		fprintf(stderr, "*** Panic: incorrect record size.\n");
		bexit(EXIT_FAILURE);
	}

	printf("\n--------------------------------------------------\n");
	printf("|                    Rolodeck                    |\n");
	printf("--------------------------------------------------\n\n");

	/* set sort field */
	sf = RD_CONTACT;

	/* display menu */
	putmenu(sf);

	/* main loop */
	memset(&rd, 0, sizeof(rd));
	while (1) {
		printf("Enter selection:  ");
		fgets(buf, sizeof(buf), stdin);
		cvtss(buf, buf, CVT_X_SP | CVT_X_CTL, sizeof(buf));
		rq = toupper(buf[0]);
		if (rq == RD_REQ_QUIT) {	/* quit rolodeck */
			break;
		}
		if (rq == '\0') {		/* default to next card */
			rq = RD_REQ_NEXT_CARD;
		}
		switch (rq) {
		case RD_REQ_NEXT_CARD:	/* next card */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			/* use unique key field to set record cursor */
			found = cbkeysrch(cbp, RD_CONTACT, rd.rd_contact);
			if (found == -1) {
				fprintf(stderr, "*** Error %d searching for key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* align cursor of sort key */
			if (cbkeyalign(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d aligning key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (found == 1) {
				/* advance key (and rec) cursor 1 position */
				if (cbkeynext(cbp, sf) == -1) {
					fprintf(stderr, "*** Error %d finding next card.\n", errno);
					bexit(EXIT_FAILURE);
				}
			}
			if (cbrcursor(cbp) == NULL) {
				printf("End of deck.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbgetr(cbp, &rd) == -1) {
				fprintf(stderr, "*** Error %d reading card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_NEXT_CARD: */
		case RD_REQ_PREV_CARD:	/* previous card */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			/* use unique key field to set record cursor */
			found = cbkeysrch(cbp, RD_CONTACT, rd.rd_contact);
			if (found == -1) {
				fprintf(stderr, "*** Error %d searching for key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* align cursor of sort key */
			if (cbkeyalign(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d aligning key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* retreat key (and rec) cursor 1 position */
			if (cbkeyprev(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d finding previous card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbrcursor(cbp) == NULL) {
				printf("Beginning of deck.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbgetr(cbp, &rd) == -1) {
				fprintf(stderr, "*** Error %d reading card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_PREV_CARD: */
		case RD_REQ_FIRST_CARD:	/* first card */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if(rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbkeyfirst(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d finding first card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbgetr(cbp, &rd) == -1) {
				fprintf(stderr, "*** Error %d reading card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_FIRST_CARD: */
		case RD_REQ_LAST_CARD:	/* last card */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbkeylast(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d finding last card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbgetr(cbp, &rd) == -1) {
				fprintf(stderr, "*** Error %d reading card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_LAST_CARD: */
		case RD_REQ_INS_CARD:	/* insert new card */
			getcard(&rd);
			if (strlen(rd.rd_contact) == 0) {
				printf("Contact name cannot be blank.  Card not inserted.\n\n");
				memset(&rd, 0, sizeof(rd));
				continue;
			}
			if (rdlock(cbp, CB_WRLCK) == -1) {
				fprintf(stderr, "*** Error %d write locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbinsert(cbp, &rd) == -1) {
				if (errno == CBEDUP) {
					printf("%s is already in the rolodeck.\n\n", rd.rd_contact);
					if (rdlock(cbp, CB_UNLCK) == -1) {
						fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
						bexit(EXIT_FAILURE);
					}
					continue;
				}
				fprintf(stderr, "*** Error %d inserting card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_INS_CARD: */
		case RD_REQ_DEL_CARD:	/* delete current card */
			if (rdlock(cbp, CB_WRLCK) == -1) {
				fprintf(stderr, "*** Error %d write locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* use unique key field to set record cursor */
			found = cbkeysrch(cbp, RD_CONTACT, rd.rd_contact);
			if (found == -1) {
				fprintf(stderr, "*** Error %d searching for key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (found == 0) {
				printf("There is no current card.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbdelcur(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d deleting current card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			lfmtofml(buf, rd.rd_contact, sizeof(buf));
			printf("%s deleted from rolodeck.\n\n", buf);
			/* load rd with card following one deleted */
			if (cbrcursor(cbp) == NULL) {
				memset(&rd, 0, sizeof(rd));
			} else {
				if (cbgetr(cbp, &rd) == -1) {
					fprintf(stderr, "*** Error %d reading card.\n", errno);
					bexit(EXIT_FAILURE);
				}
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			break;	/* case RD_REQ_DEL_CARD: */
		case RD_REQ_SRCH_CARD:	/* search for card */
			if (sf == RD_CONTACT) {
				printf("Enter contact name:  ");
			} else {
				printf("Enter company name:  ");
			}
			if (fgets(buf, sizeof(buf), stdin) == NULL) {
				fprintf(stderr, "*** Error %d reading input.\n");
				bexit(EXIT_FAILURE);
			}
			cvtss(buf, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(buf));
			if (sf == RD_CONTACT) {
				fmltolfm(buf, buf, sizeof(buf));
			}
			/* don't lock until after user input */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			found = cbkeysrch(cbp, sf, buf);
			if (found == -1) {
				fprintf(stderr, "*** Error %d searching for key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (found == 0) {
				printf("%s not found.\n\n", buf);
				if (cbrcursor(cbp) == NULL) {
					if (cbkeylast(cbp, sf) == -1) {
						fprintf(stderr, "*** Error %d finding last card.\n", errno);
						bexit(EXIT_FAILURE);
					}
				}
			}
			if (cbgetr(cbp, &rd) == -1) {
				fprintf(stderr, "*** Error %d reading card.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			putcard(&rd);
			break;	/* case RD_REQ_SRCH_CARD: */
		case RD_REQ_TOG_SORT:	/* toggle sort field */
			switch (sf) {
			case RD_CONTACT:
				sf = RD_COMPANY;
				break;
			case RD_COMPANY:
				sf = RD_CONTACT;
				break;
			default:
				sf = RD_CONTACT;
				break;
			}
			putmenu(sf);
			break;	/* case RD_REQ_TOG_SORT: */
		case RD_REQ_CUR_CARD:	/* display current card */
			if (strlen(rd.rd_contact) == 0) {
				printf("There is no current card.\n");
				continue;
			}
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* use unique key field to set record cursor */
			found = cbkeysrch(cbp, RD_CONTACT, rd.rd_contact);
			if (found == -1) {
				fprintf(stderr, "*** Error %d searching for key.\n", errno);
				bexit(EXIT_FAILURE);
			}
			/* check if card deleted by other process */
			if (found == 0) {
				printf("There is no current card.\n\n");
			} else {
				if (cbgetr(cbp, &rd) == -1) {
					fprintf(stderr, "*** Error %d reading card.\n", errno);
					bexit(EXIT_FAILURE);
				}
				putcard(&rd);
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			break;	/* case RD_REQ_CUR_CARD: */
		case RD_REQ_ALL_CARDS:	/* display all cards */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			if (cbreccnt(cbp) == 0) {
				printf("The rolodeck is empty.\n\n");
				if (rdlock(cbp, CB_UNLCK) == -1) {
					fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
					bexit(EXIT_FAILURE);
				}
				continue;
			}
			if (cbkeyfirst(cbp, sf) == -1) {
				fprintf(stderr, "*** Error %d setting key cursor.\n", errno);
				bexit(EXIT_FAILURE);
			}
			while (cbkcursor(cbp, sf) != NULL) {
				if (cbgetr(cbp, &rd) == -1) {
					fprintf(stderr, "*** Error %d reading card.\n", errno);
					bexit(EXIT_FAILURE);
				}
				putcard(&rd);
				if (cbkeynext(cbp, sf) == -1) {
					fprintf(stderr, "*** Error %d finding next card.\n", errno);
					bexit(EXIT_FAILURE);
				}
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			break;	/* case RD_REQ_ALL_CARDS: */
		case RD_REQ_IMPORT:	/* import cards */
			if (rdlock(cbp, CB_WRLCK) == -1) {
				fprintf(stderr, "*** Error %d write locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			printf("Importing cards from %s....\n", EXPFILE);
			if (cbimport(cbp, EXPFILE) == -1) {
				if (errno == ENOENT) {
					printf("Printable file not found.\n\n");
				} else if (errno == CBEDUP) {
					printf("Duplicate card encountered.  Import terminated.\n\n");
				} else {
					fprintf(stderr, "*** Error %d importing rolodeck from %s.\n", errno, EXPFILE);
					bexit(EXIT_FAILURE);
				}
			} else {
				printf("Import complete.\n\n");
			}
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			break;	/* cbase RD_REQ_IMPORT: */
		case RD_REQ_EXPORT:	/* export cards */
			if (rdlock(cbp, CB_RDLCK) == -1) {
				fprintf(stderr, "*** Error %d read locking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			printf("Exporting cards to %s....\n", EXPFILE);
			if (cbexport(cbp, EXPFILE) == -1) {
				fprintf(stderr, "*** Error %d exporting rolodeck to %s.\n", errno, EXPFILE);
				bexit(EXIT_FAILURE);
			}
			printf("Export complete.\n\n");
			if (rdlock(cbp, CB_UNLCK) == -1) {
				fprintf(stderr, "*** Error %d unlocking rolodeck.\n", errno);
				bexit(EXIT_FAILURE);
			}
			break;	/* cbase RD_REQ_EXPORT: */
		case RD_REQ_HELP:	/* help */
			putmenu(sf);
			continue;
			break;	/* case RD_REQ_HELP: */
		default:
			putchar('\a');	/* beep */
			continue;
			break;	/* default: */
		}
	}

	/* close cbase */
	if (cbclose(cbp) == -1) {
		fprintf(stderr, "*** Error %d closing rolodeck.\n", errno);
		bexit(EXIT_FAILURE);
	}

	exit(EXIT_SUCCESS);
}

/* rdlock:  lock rolodeck */
int rdlock(cbp, ltype)
cbase_t *cbp;
int ltype;
{
	int i = 0;

	for (i = 0; i < LMAXTRIES; i++) {
		if (cblock(cbp, ltype) == -1) {
			if (errno == EAGAIN) {
				continue;
			}
			return -1;
		} else {
			errno = 0;
			return 0;
		}
	}

	errno = EAGAIN;
	return -1;
}

/* putmenu:  display menu */
int putmenu(sf)
int sf;
{
	printf("-----------------------MENU-----------------------\n");
	printf("| N - Next card          P - Previous card       |\n");
	printf("| F - First card         L - Last card           |\n");
	printf("| I - Insert new card    D - Delete current card |\n");
	printf("| S - Search for card    T - Toggle sort field   |\n");
	printf("| C - display Current    A - display All cards   |\n");
	printf("| M - iMport cards       X - eXport cards        |\n");
	printf("| H - Help (menu)        Q - Quit                |\n");
	printf("--------------------------------------------------\n");
	printf("current sort field:  ");
	switch (sf) {
	case RD_CONTACT:
		printf("contact");
		break;
	case RD_COMPANY:
		printf("company");
		break;
	default:
		printf("none");
		break;
	}
	printf(".\n\n");

	return 0;
}

/* getcard:  input card */
int getcard(rdp)
rolodeck_t *rdp;
{
	char buf[256];
	int i = 0;

	memset(rdp, 0, sizeof(*rdp));

	printf("Contact:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(buf, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(buf));
	fmltolfm(rdp->rd_contact, buf, sizeof(rdp->rd_contact));
	printf("Title:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_title, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(rdp->rd_title));
	printf("Company:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_company, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(rdp->rd_company));
	printf("Street Address (2 lines):\n");
	for (i = 0; i < 2; i++) {
		fgets(buf, sizeof(buf), stdin);
		cvtss(buf, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(buf));
		sprintf(rdp->rd_addr + 40 * i, "%-40.40s", buf);
	}
	printf("City:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_city, buf, CVT_X_LEADSP | CVT_X_TRAILSP | CVT_ONE_SP | CVT_X_CTL, sizeof(rdp->rd_city));
	printf("State:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_state, buf, CVT_X_SP | CVT_X_CTL, sizeof(rdp->rd_state));
	printf("Zip Code:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_zip, buf, CVT_X_SP | CVT_X_CTL, sizeof(rdp->rd_zip));
	printf("Phone:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_phone, buf, CVT_X_SP | CVT_X_CTL, sizeof(rdp->rd_phone));
	printf("Extension:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_ext, buf, CVT_X_SP | CVT_X_CTL, sizeof(rdp->rd_ext));
	printf("Fax:  ");
	fgets(buf, sizeof(buf), stdin);
	cvtss(rdp->rd_fax, buf, CVT_X_SP | CVT_X_CTL, sizeof(rdp->rd_fax));
	printf("Notes (4 lines):\n");
	for (i = 0; i < 4; i++) {
		fgets(buf, sizeof(buf), stdin);
		cvtss(buf, buf, CVT_X_CTL, sizeof(buf));
		sprintf(rdp->rd_notes + 40 * i, "%-40.40s", buf);
	}

	return 0;
}

/* putcard:  display card */
int putcard(rdp)
rolodeck_t *rdp;
{
	char name[sizeof(rdp->rd_contact)];

	lfmtofml(name, rdp->rd_contact, sizeof(name));
	printf("--------------------CARD--------------------\n");
	printf("| %-40.40s |\n", name);
	printf("| %-40.40s |\n", rdp->rd_title);
	printf("| %-40.40s |\n", rdp->rd_company);
	printf("| %-40.40s |\n", rdp->rd_addr);
	printf("| %-40.40s |\n", rdp->rd_addr + 40);
	printf("| %-25.25s, %-2.2s %-10.10s |\n", rdp->rd_city, rdp->rd_state, rdp->rd_zip);
	printf("| %-12.12s x%-4.4s  fax %-12.12s     |\n", rdp->rd_phone, rdp->rd_ext, rdp->rd_fax);
	printf("| %-40.40s |\n", rdp->rd_notes);
	printf("| %-40.40s |\n", rdp->rd_notes + 40);
	printf("| %-40.40s |\n", rdp->rd_notes + 80);
	printf("| %-40.40s |\n", rdp->rd_notes + 120);
	printf("--------------------------------------------\n");

	return 0;
}
